#ifndef _fawk__defines_h_
#define _fawk__defines_h_

typedef short fawk_int_t;
#define fawk_chr yyctx->chr
#define fawk_val yyctx->val
#define fawk_lval yyctx->lval
#define fawk_stack yyctx->stack
#define fawk_debug yyctx->debug
#define fawk_nerrs yyctx->nerrs
#define fawk_errflag yyctx->errflag
#define fawk_state yyctx->state
#define fawk_yyn yyctx->yyn
#define fawk_yym yyctx->yym
#define fawk_jump yyctx->jump
#line 2 "fawk.y"
/*
libfawk - A function-only AWK dialect

Copyright (c) 2017..2020 Tibor 'Igor2' Palinkas. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

  * 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
  * 3. Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Project page: http://repo.hu/projects/libfawk
Source code: svn://repo.hu/libfawk/trunk
Contact the author: http://igor2.repo.hu/contact.html
*/

#include <libfawk/config.h>
#include <stdlib.h>
#include <libfawk/libfawk.h>
#include <libfawk/parser.h>
#include "fawk.y.tab.h"
#include "parser_fawk.h"
#include <libfawk/symtab.h>
#include <libfawk/compile.h>
#include <libfawk/execute.h>
#line 46 "fawk.y"
typedef union fawk_tokunion_u {
	char *str;
	fawk_num_t num;
} fawk_tokunion_t;
typedef fawk_tokunion_t fawk_STYPE;


#define T_FUNCTION 257
#define T_EQ 258
#define T_NEQ 259
#define T_GTEQ 260
#define T_LTEQ 261
#define T_PLEQ 262
#define T_MIEQ 263
#define T_MUEQ 264
#define T_DIEQ 265
#define T_MOEQ 266
#define T_NIL 267
#define T_IF 268
#define T_ELSE 269
#define T_FOR 270
#define T_IN 271
#define T_DO 272
#define T_WHILE 273
#define T_RETURN 274
#define T_ID 275
#define T_STRING 276
#define T_NUM 277
#define T_OR 278
#define T_AND 279
#define T_MM 280
#define T_PP 281
#define fawk_ERRCODE 256

#ifndef fawk_INITSTACKSIZE
#define fawk_INITSTACKSIZE 200
#endif

typedef struct {
	unsigned stacksize;
	fawk_int_t *s_base;
	fawk_int_t *s_mark;
	fawk_int_t *s_last;
	fawk_STYPE *l_base;
	fawk_STYPE *l_mark;
#if fawk_DEBUG
	int debug;
#endif
} fawk_STACKDATA;

typedef struct {
	int errflag;
	int chr;
	fawk_STYPE val;
	fawk_STYPE lval;
	int nerrs;
	int yym, yyn, state;
	int jump;
	int stack_max_depth;
	int debug;

	/* variables for the parser stack */
	fawk_STACKDATA stack;
} fawk_yyctx_t;

typedef enum { fawk_RES_NEXT, fawk_RES_DONE, fawk_RES_ABORT } fawk_res_t;

FAWK_API int fawk_parse_init(fawk_yyctx_t *yyctx);
FAWK_API fawk_res_t fawk_parse(fawk_yyctx_t *yyctx, fawk_ctx_t *ctx, int tok, fawk_STYPE *lval);
FAWK_API void fawk_error(fawk_ctx_t *ctx, fawk_STYPE tok, const char *msg);


#endif /* _fawk__defines_h_ */
